#include  "header.h"

//Preparation of "LIST2" in the dialog box.
void LIST2_prep()
{
	LVCOLUMN lvcol;	//A structure that contains the information of columns in "LIST2".

	int CX[] = { C_WIDTH0, C_WIDTH1, C_WIDTH2, C_WIDTH3, C_WIDTH4, C_WIDTH5, C_WIDTH6, C_WIDTH7, C_WIDTH8, C_WIDTH9 };	//The widths of the 10 coloumns defined in header.h.
	LPTSTR strItem0[] = { (LPTSTR)TEXT(""), (LPTSTR)TEXT("SER file name"), (LPTSTR)TEXT("frames"), (LPTSTR)TEXT("start"), (LPTSTR)TEXT("stop"), (LPTSTR)TEXT("Max"), (LPTSTR)TEXT("pack."), (LPTSTR)TEXT("BLUE"), (LPTSTR)TEXT("RED"), (LPTSTR)TEXT("Brt") };	//Header names of the 10 colomns.

	InitCommonControls();
	DWORD dwStyle = ListView_GetExtendedListViewStyle( hList2 );	//"hList2" is the handle of "LIST2".
	dwStyle |= LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES;				//Set two options to a ListViewStyle.
	ListView_SetExtendedListViewStyle( hList2, dwStyle );			//Set the ListViewStyle to "hList2".

	lvcol.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvcol.fmt = LVCFMT_LEFT;			//Align text to the left.
	for( int ic = 0; ic < 10; ic++ ) {	//For 10 columns;
		lvcol.cx = CX[ic];				//Set the column width.
		lvcol.pszText = strItem0[ic];	//Set the header name for the column.
		lvcol.iSubItem = ic;			//Set subitem number for the column.
		ListView_InsertColumn( hList2, ic, &lvcol );	//Insert a column in "LIST2" whose handle is "hList2".
	}

	return;
}

//Add informations and settings of a selected SER file in "LIST1" (ListBox) to a new line in "LIST2".
void  LIST2_add()
{
	char buffer[256];
	LVITEM lv_item;
	lv_item.mask = LVIF_TEXT;	//Make ".pszText" an active member.
	lv_item.iItem = list_file;	//"list_file" is the number of SER files in "LIST2".

	//An item for a marker which indicates that a SER file of this line in "LIST2" is now displayed.
	lv_item.pszText = (LPTSTR)"";					//Make "lv_item.pszText" blank.
	lv_item.iSubItem = 0;							//"lv_item.iSubItem" is the subitem# for a column.  "0" corresponds to the left end.
	ListView_InsertItem( hList2, &lv_item );		//For "lv_item.iSubItem" = 0, we use "ListView_InsertItem".

	//A SER file name.
	lv_item.pszText = list_fname;					//An SER file name with its extension.
	lv_item.iSubItem = 1;							//"1" corresponds to the second from the left.
	ListView_SetItem( hList2, &lv_item );			//For non zero "lv_item.iSubItem", we use "ListView_SetItem".

	//The number of frames in the SER file.
	wsprintf(buffer, "%d", FrameCount );
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 2;
	ListView_SetItem(hList2, &lv_item);

	//The first frame# to be scanned (default value).
	lv_item.pszText = (LPTSTR)"0";
	lv_item.iSubItem = 3;
	ListView_SetItem( hList2, &lv_item );

	//The last frame# to be scanned (default value).
	wsprintf(buffer, "%d", FrameCount - 1 );		//(number of frames in the SER file) - 1.
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 4;
	ListView_SetItem(hList2, &lv_item);

	//The upper limit of the number of frames that include detections (default value).
	int c = FrameCount / 100 + 1;							//"FrameCount" is the number of frames in the SER file.
	if( c > FRAMES_DETECTED_MAX ) c = FRAMES_DETECTED_MAX;	//see header.h for "FRAMES_DETECTED_MAX".
	wsprintf(buffer, "%d", c );
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 5;
	ListView_SetItem(hList2, &lv_item);

	//Packet size, that is, the number of frames in a packet (default value).
	wsprintf(buffer, "%d", PACKET );				//See header.h for "PACKET".
	if( FrameCount < PACKET ) wsprintf( buffer, "%d", FrameCount );
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 6;
	ListView_SetItem(hList2, &lv_item);

	//A pixel value that defines dark pixels (default value).
	wsprintf(buffer, "%d", BLUE_VALUE );
	if( PixelDepthPerPlane == 8 ) wsprintf(buffer, "%d", 0 );	//In the case the grey scale of the SER file is 8-bit;
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 7;
	ListView_SetItem(hList2, &lv_item);

	//A pixel value that defines bright pixels (default value).
	wsprintf(buffer, "%d", RED_VALUE  );
	if( PixelDepthPerPlane == 8 ) wsprintf(buffer, "%d", 255 );	//In the case the grey scale of the SER file is 8-bit;
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 8;
	ListView_SetItem(hList2, &lv_item);

	//SubItem No. 9 controls the brightness of the Image window and the output BMP files (default value).
	double factor = pow(2.0, (double)(bit_shift - 8)) * 1000.0;		//Calculate the maximum of a 16-bit value with bit_shift.
	wsprintf(buffer, "%d", (int)(factor + 0.5));
	if( PixelDepthPerPlane == 8 ) wsprintf(buffer, "%d", 1000 );	//In the case the grey scale of the SER file is 8-bit, no brightness control is applied ("1000" corresponds to factor 1).
	lv_item.pszText = buffer;				
	lv_item.iSubItem = 9;
	ListView_SetItem(hList2, &lv_item);

	return;
}

//Make "SELECTED" and "FOCUSED" states OFF for all SER files in "LIST2".
void LIST2_deselect()
{
	UINT uiState;
	for( int i = 0; i < list_file; i++ ) {
		uiState = ListView_GetItemState( hList2, i, LVIS_SELECTED | LVIS_FOCUSED );	//Check the state for a SER file of the item# i.
		if ( (uiState & LVIS_SELECTED) || (uiState & LVIS_FOCUSED) ) {		//If selected or focused;
			ListView_SetItemState( hList2, i, 0, LVIS_SELECTED );			//Make "SELECTED" states OFF.
			ListView_SetItemState( hList2, i, 0, LVIS_FOCUSED  );			//Make "FOCUSED" states OFF.
		}
	}
	return;
}

//Calculate a subitem number in "LIST2" from a double-click position.
int get_iSubItem( int x )	//"x" is the x-coordinate of the double-click position in "LIST2".
{
	int j;

	if ( x < C_WIDTH0 ) j = 0;
	else if ( x < C_WIDTH0+C_WIDTH1 ) j = 1;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2 ) j = 2;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3 ) j = 3;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3+C_WIDTH4 ) j = 4;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3+C_WIDTH4+C_WIDTH5 ) j = 5;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3+C_WIDTH4+C_WIDTH5+C_WIDTH6 ) j = 6;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3+C_WIDTH4+C_WIDTH5+C_WIDTH6+C_WIDTH7 ) j = 7;
	else if ( x < C_WIDTH0+C_WIDTH1+C_WIDTH2+C_WIDTH3+C_WIDTH4+C_WIDTH5+C_WIDTH6+C_WIDTH7+C_WIDTH8 ) j = 8;
	else j = 9;

	return j;
}

//Get a value in a cell in "LIST2".
int get_item_value( int item, int subitem )	//"item" corresponds to a row, and "subitem" corresponds to a column in "LIST2".
{
	char buffer[256];
	int  item_int;				//The value in a cell specified by "item" and "subitem".

	LVITEM lv_item;
	lv_item.mask = LVIF_TEXT;	//Make ".pszText" an active member.

	lv_item.iItem = item;
	lv_item.iSubItem = subitem;
	lv_item.pszText = buffer;	//"buffer" gets the text information in "lv_item.pszText".
	lv_item.cchTextMax = 256;	//The maximum text length.
	ListView_GetItem( hList2, &lv_item );	//Get a text information for item#=item and subitem#=subitem.

	item_int = atoi( buffer );	//Convert the text information to an integer.
	return item_int;
}

//Show "1" in the left end column for a specified item number in "LIST2", and show blank for the others.
void LIST2_set_checkmark( int item )
{
	LVITEM lv_item;
	lv_item.mask = LVIF_TEXT;	//Make ".pszText" an active member.
	lv_item.iSubItem = 0;		//"0" corresponds to the left end column.

	for( int i = 0; i < list_file; i++ ) {			//For all SER files in "LIST2";
		lv_item.iItem = i;							//"lv_item.iItem" corresponds to a SER file.
		if( i == item ) {
			lv_item.pszText = (LPTSTR)"1";			//Show "1" for the specified item number.
			ListView_SetItem( hList2, &lv_item );
		}
		else {
			lv_item.pszText = (LPTSTR)"";			//Show blank for the others.
			ListView_SetItem( hList2, &lv_item );
		}
	}
	return;
}

//Return the item number (SER file number) in "LIST2" for which the left end column shows "1".
int LIST2_get_checkmarked_item()
{
	char buffer[256];
	LVITEM lv_item;
	lv_item.mask = LVIF_TEXT;	//Make ".pszText" an active member.
	lv_item.iSubItem = 0;		//"0" corresponds to the left end column.

	for( int i = 0; i < list_file; i++ ) {
		lv_item.iItem = i;
		lv_item.pszText = buffer;				//"buffer" get the text information in "lv_item.pszText".
		lv_item.cchTextMax = 256;				//The maximum text length.
		ListView_GetItem( hList2, &lv_item );	//Get a text information for item#=i and subitem#=0.
		if( atoi( buffer ) == 1 ) return i;		//Convert the text to integer, return the item# if it is 1.
	}
	return -1;
}

//Return FALSE if a new value typed in a small DialogBox for changing parameters is not appropriate.
BOOL check_start_end_packet( int item, int subitem )
{
	//Present values.
	int total_frame2 = get_item_value( item, 2 );	//The number of frames in a SER file.
	int start2       = get_item_value( item, 3 );	//Frame# in a SER file at which the scan start.
	int end2         = get_item_value( item, 4 );	//Frame# in a SER file at which the scan stop.
	int packet_size2 = get_item_value( item, 6 );	//The number of frames in a packet.

	//"buf_for_dlg2" is a new value (text) typed in a small DialogBox for changing parameters.
	int item_int = atoi( buf_for_dlg2 );	//"atoi" converts (text) to (integer).
	
	switch( subitem ) {
		case 3:	//In the case of the start frame#;
			if( (end2 - item_int + 1) < packet_size2 ) return FALSE;	//The new start frame# is too close to the stop frame#.
			break;
		case 4:	//In the case of the stop frame#;
			if( item_int > total_frame2 - 1 ) return FALSE;				//The new stop frame# is too large.
			if( (item_int - start2 + 1) < packet_size2 ) return FALSE;	//The new stop frame# is too close to the start frmae#.
			break;
		case 6:	//In the case of the packet size;
			if( (end2 - start2 + 1) < item_int ) return FALSE;			//The new packet size is too large.
			break;
	}

	return TRUE;
}

//Make a correction for "c_frame" if it is not appropriate.
int c_frame_check( int item )		//"item" corresponds to a SER file.
{
	int  c_frame2;					//"c_frame" value after one of the following corrections.

	int max_frame = get_item_value( item, 2 ) - 1;		//"item" corresponds to a SER file, and "2" corresponds to the number of frames in the SER file.

	if ( c_frame > max_frame ) c_frame2 = max_frame;	//In the case of too large "c_frame";
	else if ( c_frame < 0 )    c_frame2 = 0;			//In the case of negative "c_frame";
	else                       c_frame2 = c_frame;		//In the case "c_frame" value is appropriate;

	return c_frame2;
}

//Read parameters for a scan from the DialogBox.
void read_parameter( int i_scan )
{
	BOOL check;

	total_frame  = get_item_value( i_scan, 2 ); //The number of frames in a SER file.
	start		 = get_item_value( i_scan, 3 ); //Frame# in a SER file at which the scan start.
	end          = get_item_value( i_scan, 4 ); //Frame# in a SER file at which the scan stop.
	detect_limit = get_item_value( i_scan, 5 );	//The upper limit of number of frames that include detections.
	packet_size  = get_item_value( i_scan, 6 ); //The number of frames in a packet.
	blue_val     = get_item_value( i_scan, 7 );	//A brightness criterion for dark pixels.
	red_val      = get_item_value( i_scan, 8 );	//A brightness criterion for bright pixels.
	mag_brt      = get_item_value( i_scan, 9 );	//"mag_brt" controls the brightness of the Image window and the output BMP files (8-bit brightness for display = pixel value * mag_brt/1000).
	threshold    = GetDlgItemInt( hDlg, IDC_EDIT_THRESHOLD0, &check, TRUE );	//Initial threshold for detections.
	d_threshold  = GetDlgItemInt( hDlg, IDC_EDIT_dTHRESHOLD, &check, TRUE );	//The step of the threshold value.
	hidari       = GetDlgItemInt( hDlg, IDC_EDIT_HIDARI, &check, TRUE );		//An additional brightness threshold for the left adjacent pixel (% of the detected peak value).
	migi         = GetDlgItemInt( hDlg, IDC_EDIT_MIGI,   &check, TRUE );		//right adjacent.
	ue           = GetDlgItemInt( hDlg, IDC_EDIT_UE,     &check, TRUE );		//upper adjacent.
	shita        = GetDlgItemInt( hDlg, IDC_EDIT_SHITA,  &check, TRUE );		//lower adjacent.
	mask_manual_TL[0].x = GetDlgItemInt( hDlg, IDC_EDIT_TL_X1,  &check, TRUE );	//The x-coordinate of the upper-left corner of a mask area.
	mask_manual_TL[0].y = GetDlgItemInt( hDlg, IDC_EDIT_TL_Y1,  &check, TRUE );
	mask_manual_BR[0].x = GetDlgItemInt( hDlg, IDC_EDIT_BR_X1,  &check, TRUE );
	mask_manual_BR[0].y = GetDlgItemInt( hDlg, IDC_EDIT_BR_Y1,  &check, TRUE );	//The y-coordinate of the lower-right corner of a mask area.
	mask_manual_TL[1].x = GetDlgItemInt( hDlg, IDC_EDIT_TL_X2,  &check, TRUE );
	mask_manual_TL[1].y = GetDlgItemInt( hDlg, IDC_EDIT_TL_Y2,  &check, TRUE );
	mask_manual_BR[1].x = GetDlgItemInt( hDlg, IDC_EDIT_BR_X2,  &check, TRUE );
	mask_manual_BR[1].y = GetDlgItemInt( hDlg, IDC_EDIT_BR_Y2,  &check, TRUE );
	mask_manual_TL[2].x = GetDlgItemInt( hDlg, IDC_EDIT_TL_X3,  &check, TRUE );
	mask_manual_TL[2].y = GetDlgItemInt( hDlg, IDC_EDIT_TL_Y3,  &check, TRUE );
	mask_manual_BR[2].x = GetDlgItemInt( hDlg, IDC_EDIT_BR_X3,  &check, TRUE );
	mask_manual_BR[2].y = GetDlgItemInt( hDlg, IDC_EDIT_BR_Y3,  &check, TRUE );
	mask_manual_TL[3].x = GetDlgItemInt( hDlg, IDC_EDIT_TL_X4,  &check, TRUE );
	mask_manual_TL[3].y = GetDlgItemInt( hDlg, IDC_EDIT_TL_Y4,  &check, TRUE );
	mask_manual_BR[3].x = GetDlgItemInt( hDlg, IDC_EDIT_BR_X4,  &check, TRUE );
	mask_manual_BR[3].y = GetDlgItemInt( hDlg, IDC_EDIT_BR_Y4,  &check, TRUE );

	mask_area1 = GetDlgItemInt( hDlg, IDC_EDIT_MASK_AREA1, &check, TRUE );	//"mask_area1" defines the size of the mask area, (the side length of a square - 1)/2, for bright pixels.
	mask_area2 = GetDlgItemInt( hDlg, IDC_EDIT_MASK_AREA2, &check, TRUE );	//For dark pixels.
	mask_area3 = GetDlgItemInt( hDlg, IDC_EDIT_MASK_AREA3, &check, TRUE );	//For stars.
	star_size = GetDlgItemInt( hDlg, IDC_EDIT_STAR_SEEING,  &check, TRUE );	//"star_size" defines the square around a star.
	star_threshold = GetDlgItemInt( hDlg, IDC_EDIT_STAR_THRESHOLD,  &check, TRUE );	//A threshold for stellar image detections multiplied by 10.

	scatter_i  = GetDlgItemInt( hDlg, IDC_EDIT_SCATTER_i,  &check, TRUE );	//"scatter_i" is the upper limit of the scatter of the y-coordinates of the detected points.
	scatter_j  = GetDlgItemInt( hDlg, IDC_EDIT_SCATTER_j,  &check, TRUE );

	return;
}

//Print results of a scan of a SER file to a file.
void result_txtout1( char *str_header )
{
	fprintf( fp2, "--------------SER File Header Information--------------\n" );
	fprintf( fp2, str_header );													//"str_header" contains header records of a SER file.
	fprintf( fp2, "--------------Parameters for detection-----------------\n" );
	fprintf( fp2, "start_frame# = %d  end_frame# = %d  packet_size = %dframes  brightness = %d\n", start, end, packet_size, mag_brt );
	fprintf( fp2, "Additional criteria: T = %d%% B = %d%% R = %d%% L = %d%%\n", ue, shita, migi, hidari );

	fprintf( fp2, "Manual Mask 0: UL_x = %d, UL_y = %d, LR_x = %d, LR_y = %d\n", mask_manual_TL[0].x, mask_manual_TL[0].y, mask_manual_BR[0].x, mask_manual_BR[0].y );
	fprintf( fp2, "Manual Mask 1: UL_x = %d, UL_y = %d, LR_x = %d, LR_y = %d\n", mask_manual_TL[1].x, mask_manual_TL[1].y, mask_manual_BR[1].x, mask_manual_BR[1].y );
	fprintf( fp2, "Manual Mask 2: UL_x = %d, UL_y = %d, LR_x = %d, LR_y = %d\n", mask_manual_TL[2].x, mask_manual_TL[2].y, mask_manual_BR[2].x, mask_manual_BR[2].y );
	fprintf( fp2, "Manual Mask 3: UL_x = %d, UL_y = %d, LR_x = %d, LR_y = %d\n", mask_manual_TL[3].x, mask_manual_TL[3].y, mask_manual_BR[3].x, mask_manual_BR[3].y );

	if( Button_GetCheck( hCheck_mask_bright ) == BST_CHECKED ) fprintf( fp2, "Mask around RED pixels ONF  Mask size %d\n", mask_area1 );
	else fprintf( fp2, "Mask around RED pixels OFF\n" );

	if( Button_GetCheck( hCheck_mask_dark   ) == BST_CHECKED ) fprintf( fp2, "Mask around BLUE pixels ONFMask size %d\n", mask_area2 );
	else fprintf( fp2, "Mask around BLUE pixels OFF\n" );

	if( Button_GetCheck( hCheck_mask_star   ) == BST_CHECKED ) fprintf( fp2, "Mask stars ONFStar size %d Star threshold %d Mask size %d\n", star_size, star_threshold, mask_area3 );
	else fprintf( fp2, "Mask stars OFF\n" );

	fprintf( fp2, "Pixel value for BLUE = %d, Pixel value for RED = %d\n", blue_val, red_val );

	fprintf( fp2, "Acceptable ranges for the scatter of the detected points: %d(vertical), %d(horizontal)\n", scatter_i, scatter_j );

	return;
}

//Print columun headers for the results of a scan for each threshold value.
void result_txtout2()
{
	fprintf( fp2, "\n------------Detection-----------------------------------------------" );
	fprintf( fp2, "\nThreshold: %dsigma", threshold );
	fprintf( fp2, "\n%-10s%-10s%-10s%-10s%-10s%-10s%-10s", "No.","Frame","x", "y", "count", "Average", "Standard deviation" );
	return;
}
