#include  "header.h"

//Get a path name of a file.
char* file_open()
{
	OPENFILENAME	fname;
	static char		fn[256];
	static char		filefilter[] = "ser files(*.ser)\0*.ser\0"
	 							   "all files(*.*)\0*.*\0\0";

	memset( &fname, 0, sizeof(OPENFILENAME) );
	fname.lStructSize    = sizeof(OPENFILENAME);
	fname.lpstrFilter    = filefilter;
	fname.nFilterIndex   = 1;
	fname.lpstrFile      = fn;
	fname.nMaxFile       = sizeof(fn);
	fname.Flags          = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

	if( !GetOpenFileName( &fname ) )  return NULL;

	return fn;
}

//Read a header(178 Byte) of a SER file, and get the width, height, grey scale, and number of frames in the file.
char* read_header( FILE *fp1 )
{
	char cbuf[100]={};
	int ibuf;
	//unsigned long long ibuf64;
	long long ibuf64;
	const int strsize = 1000;
	static char str[1000];				//Without "static", "str[1000]" changes unexpectedly.
	char tmp[100];

	fseek ( fp1,  0, SEEK_SET );
	fread ( &cbuf, 14, 1, fp1 );		//Read the first 14 bytes in the header that contain FileID.
	wsprintf ( tmp, "FileID(14byte)F%s\x0d\x0a", cbuf );	//"\x0d\x0a" is equvalent to "\n" and means CR+LF.
	strcpy_s ( str, strsize, tmp );							//Copy "tmp" to the top of "str".
	fread ( &ibuf, 4,  1, fp1 );
	wsprintf ( cbuf, "LuID(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );						//Add "cbuf" to "str".

	fread ( &ibuf, 4,  1, fp1 );
	wsprintf ( cbuf, "ColorID(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf, 4,  1, fp1 );
	wsprintf ( cbuf, "LittleEndian(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf, 4,  1, fp1 );
	ImageWidth = ibuf;										//Get the width of the frame.
	wsprintf ( cbuf, "ImageWidth(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf, 4,  1, fp1 );
	ImageHeight = ibuf;										//Get the height of the frame.
	wsprintf ( cbuf, "ImageHeight(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf, 4,  1, fp1 );
	PixelDepthPerPlane = ibuf;								//Get the grey-scale information(16-bit or 8-bit) of the frame.
	wsprintf ( cbuf, "PixelDepthPerPlane(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf, 4,  1, fp1 );
	FrameCount = ibuf;										//Get the number of frames in the SER file.
	wsprintf ( cbuf, "FrameCount(int)F%d\x0d\x0a", ibuf );
	strcat_s ( str, strsize, cbuf );

	fread ( &cbuf, 40,  1, fp1 );
	wsprintf ( tmp, "Observer(40byte)F%s\x0d\x0a", cbuf );
	strcat_s ( str, strsize, tmp );

	fread ( &cbuf, 40,  1, fp1 );
	wsprintf ( tmp, "Instrument(40byte)F%s\x0d\x0a", cbuf );
	strcat_s ( str, strsize, tmp );

	fread ( &cbuf, 40,  1, fp1 );
	wsprintf ( tmp, "Telescope(40byte)F%s\x0d\x0a", cbuf );
	strcat_s ( str, strsize, tmp );

	fread ( &ibuf64, 8,  1, fp1 );
	//wsprintf ( cbuf, "Datetime:%I64x\x0d\x0a", ibuf64 );
	wsprintf ( cbuf, "Datetime:%d\x0d\x0a", ibuf64 );
	strcat_s ( str, strsize, cbuf );

	fread ( &ibuf64, 8,  1, fp1 );
	//wsprintf ( cbuf, "Datetime UTC:%I64x\x0d\x0a", ibuf64 );
	wsprintf ( cbuf, "Datetime UTC:%d\x0d\x0a", ibuf64 );
	strcat_s ( str, strsize, cbuf );
	
	return str;
}

//Read a frame from a SER file, and store it in "value16".
void ser_read( FILE *fp1 )	//"fp1" is a pointer from where we read a frame data.
{
	int i, j;

	if( PixelDepthPerPlane == 16 ) {			//In the case of 16-bit ser data;
		fread( value16, sizeof(unsigned short), ImageSize, fp1 );	//Read a frame.
		for( i = 0; i < ImageHeight; i++ ) {
			for( j = 0; j < ImageWidth; j++ ) {
				value16[ImageWidth*i +j] = value16[ImageWidth*i +j] >> bit_shift;	//Bit shift for the 12 bits or 14 bits output of some cameras.
			}
		}
	}
	
	else if( PixelDepthPerPlane == 8 ) {		//In the case of 8-bit ser data;
		static unsigned char *value8;												//"value8" is for a 8-bit frame.
		value8  = (unsigned char *)malloc( ImageSize * sizeof( unsigned char ) );	//Allocate memory for "value8".
		fread( value8,  sizeof(unsigned char),  ImageSize, fp1 );					//Read a frame.
		for( i = 0; i < ImageHeight; i++ ) {
			for( j = 0; j < ImageWidth; j++ ) {
				value16[ImageWidth*i +j] = (unsigned short)value8[ImageWidth*i +j];	//Copy 8-bit data, "value8", to "value16".
			}
		}
		free( value8 );	//Free the memory for "value8".
	}
	
	else {										//In the case of neither 16-bit nor 8-bit;
		for( i = 0; i < ImageHeight; i++ ) {
			for( j = 0; j < ImageWidth; j++ ) {
				value16[ImageWidth*i +j] = 0;
			}
		}
	}

	return;
}
