#include  "header.h"

//Preparation for a display window and a BITMAP image.
void prep_for_display()
{
	RECT dlg_rect;

	ImageWidth2  = ImageWidth / disp_size;		//"ImageWidth2" is the width for display.
	ImageHeight2 = ImageHeight / disp_size;
	ImageSize = ImageWidth * ImageHeight;		//"ImageSize" is the number of pixels in a frame.

	if( value16 != NULL ) free( value16 );
	value16 = (unsigned short *)malloc( ImageSize * sizeof( unsigned short ) );		//Allocate memories for a frame data, that is, "value16" (16 bits for each pixel).

	//Creating a image window.
	DestroyWindow( g_img.hwin );						//Once, close the window.
	InitInstance0( NULL, SW_SHOWNORMAL );				//Show a window for a diaply.  See "main.cpp" for "InitInstance0".
	if( GetWindowRect( hDlg, &dlg_rect ) == FALSE ) {	//Get the position of the Dialog Box".
		MessageBox(NULL, "GetWindowRect( hDlg, &dlg_rect ) == FALSE", "prep_for_display()", MB_OK);
		return;
	}

	prep_bmp_ih();		//Prepare BITMAP information header for display.

	return;
}

//Prepare BITMAP information header for display and file save.
void prep_bmp_ih()
{
	g_img.bmp_ih.biSize = sizeof(BITMAPINFOHEADER);
	g_img.bmp_ih.biWidth = ImageWidth;
	g_img.bmp_ih.biHeight = -ImageHeight;	//"-" (negative) value set the origin of the coordinates to the upper-left corner of an image.
	g_img.bmp_ih.biPlanes = 1;
	g_img.bmp_ih.biBitCount = 32;			//8 bits x 3 colour (RGB) + dammy (8 bits).
	g_img.bmp_ih.biCompression = BI_RGB;
	g_img.bmp_ih.biSizeImage = ImageWidth * ImageHeight * 4; //Width * Height * BytesPerPixel(4 bytes).
	g_img.bmp_ih.biXPelsPerMeter = 3704;	//Pixels/meter: If you use an EIZO FlexScan EV2455 24.1" display, 1000mm/0.27mm=3704.
	g_img.bmp_ih.biYPelsPerMeter = 3704;
	g_img.bmp_ih.biClrUsed = 0;
	g_img.bmp_ih.biClrImportant = 0;

	if( g_img.lpBmpData != NULL ) free( g_img.lpBmpData );
	g_img.lpBmpData = (BYTE *)malloc( g_img.bmp_ih.biSizeImage );	//Allocating memory for an image.

	return;
}

//Show a frame on a display.  "flag_blue_red" controls whether the bright and the dark pixels are shown in colour or not.
void frame_display( BOOL flag_blue_red )
{
	double bright;		//A 0.-255. value for display.
	int i, j, ij;

	for ( i = 0; i < ImageHeight; i++ ) {
		for (j = 0; j < ImageWidth; j++ ) {
			ij = i*ImageWidth + j;				//"ij" is the sequential number for a pixel in a frame.

			bright = (double)value16[ij] * (double)mag_brt/1000.;	//"mag_brt" controls the brightness of images that are displayed or saved.

			if( bright < 0.   ) bright = 0.;
			if( bright > 255. ) bright = 255.;	//"bright" must be less than 256 to be displayed or saved by 8-bit grey scale.
			//"g_img" is a structure "IMG0" defined in header.h.
			g_img.lpBmpData[ij*4+0] = (byte)bright;	//Blue
			g_img.lpBmpData[ij*4+1] = (byte)bright;	//Green
			g_img.lpBmpData[ij*4+2] = (byte)bright;	//Red
			g_img.lpBmpData[ij*4+3] = 0;			//Dummy

			if( flag_blue_red ) {	//"flag_blue_red" is an argument and controls whether the bright and the dark pixels are shown in colour or not.
				if( value16[ij] < blue_val ) {
					g_img.lpBmpData[ij*4+0] = 255;	//Dark pixels are shown by blue.
					g_img.lpBmpData[ij*4+1] = 0;
					g_img.lpBmpData[ij*4+2] = 0;
				}
				if( value16[ij] > red_val  ) {
					g_img.lpBmpData[ij*4+0] = 0;
					g_img.lpBmpData[ij*4+1] = 0;
					g_img.lpBmpData[ij*4+2] = 255;	//Bright pixels are shown by red.
				}
			}
		}
	}

	/*
	//Plot dots to 100 every other pixels vertically and horizontally.
	for ( i = 0; i < ImageHeight; i=i+100 ) {
		for (j = 0; j < ImageWidth; j=j+100 ) {
			ij = i*ImageWidth + j;

			g_img.lpBmpData[ij*4+0] =   0;	//Blue
			g_img.lpBmpData[ij*4+1] = 255;	//Green
			g_img.lpBmpData[ij*4+2] = 255;	//Red
			g_img.lpBmpData[ij*4+3] = 0;	//Dummy

		}
	}
	*/

	InvalidateRect( g_img.hwin, NULL, FALSE );

	return;
}

//When a SER file name is double-clicked in "LIST2" in the dialog box, this function displays a image.  This function return "TRUE" in the case of success.
BOOL show_file( int item, BOOL flag_blue_red )	//"item" is the item number that identifies a SER file, and "flag_blue_red" controls whether the bright and the dark pixels are shown in colour or not.
{
	errno_t error;
	int	c_frame;		//A sequential frmae nmber in a SER file.

	if( fp1 != NULL ) {	//fp1 is a pointer to FILE structure.
		fclose( fp1 );
	}
	error = fopen_s ( &fp1, file_path[item], "rb" );	//Open the "file_path[item]" in binary-reading-mode.
	if( error == 0 ) {			//In the case "fopen_s" succeeds;
		read_header ( fp1 );	//Read the header of the SER file.
		prep_for_display();		//Preparation for a display window and a BITMAP image.
		ser_read( fp1 );		//Read a frame from a SER file, and store it in "value16".

		c_frame = 0;
		SetDlgItemInt( hDlg, IDC_EDIT_FRAME, c_frame, FALSE );	//Set the "c_frame" value in the EditBox "IDC_EDIT_FRAME" in the DialogBox whose handle is "hDlg".

		blue_val = get_item_value( item, 7 );			//Get "blue_val" from the item# 7 in "LIST2" in the Dialog Box.
		red_val  = get_item_value( item, 8 );			//If the raw pixel value exceeds "red_val", it is displayed and saved by red.
		mag_brt  = get_item_value( item, 9 );			//"mag_brt" controls the brightness of images that are displayed or saved.
		frame_display( flag_blue_red );
		SetWindowText( g_img.hwin, file_path[item] );	//Chenge the name of the image window.
		return TRUE;
	}
	else {
		MessageBox( NULL, "Error in fopen_s", "show_file", MB_OK );
		return FALSE;
	}

}

//Draws a rectangular in the image window.
void DrawRect(HWND hWnd, POINTS beg, POINTS end, int fnDrawMode)
{
	HDC hdc = GetDC(hWnd);								//"hdc" is a device context of the window whose handle is hWnd.
	HPEN hPen = CreatePen(NULL, 3, RGB(255, 255, 0));	//Prepare a pen as "hPen". Its width is 3 and its color is yellow.
	SelectObject(hdc, hPen);							//Use "hPen" in "hdc".
	SetROP2(hdc, fnDrawMode);				//Set the draw mode in "hdc" as specified in "fnDrawMode". If fnDrawMode = R2_NOT, colors are inverted. If fnDrawMode = R2_COPYPEN, the color of the "hPen" is used.
	MoveToEx(hdc, end.x, end.y, NULL);		//Move the pen to "end".
	LineTo(hdc, end.x, beg.y);				//Draw a line from the present position to ("end.x", "beg.y").
	LineTo(hdc, beg.x, beg.y);
	LineTo(hdc, beg.x, end.y);
	LineTo(hdc, end.x, end.y);
	ReleaseDC(hWnd, hdc);
	return;
}
