#include  "header.h"

//Calculate a temporal average and a temporal standard deviation of each pixel over a packet of frames.
void image_ave_and_std()
{
	int i, j, ij, n;

	//Allocate memories for the calculation.
		__int64 *x_sum	= ( __int64 * )calloc( ImageSize, sizeof( __int64 ) );
		__int64 *x2_sum	= ( __int64 * )calloc( ImageSize, sizeof( __int64 ) );

	//Set "x_sum" and "x2_sum" for all pixels to zero.
	for (i = 0; i < ImageHeight; i++) {
		for (j = 0; j < ImageWidth; j++) {
			ij = ImageWidth*i + j;
			x_sum[ij]  = 0;
			x2_sum[ij] = 0;
		}
	}

	_fseeki64 ( fp1, (__int64)HEADER_SIZE + (__int64)c_frame * (__int64)ImageSize * 2, SEEK_SET );	//Set the pointer "fp1" at the top of the image data of "c_frame".

	//Add pixel values and their square together over the packet.
	for (n = 0; n < packet_size; n++){		//"packet_size" is the number of frames included in the packet.
		ser_read(fp1);						//Read pixel values of a frame in a ser file.
		for (i = 0; i < ImageHeight; i++) {
			for (j = 0; j < ImageWidth; j++) {
				ij = ImageWidth*i + j;
				x_sum[ij]  +=  (__int64)value16[ij];
				x2_sum[ij] += ((__int64)value16[ij]) * ((__int64)value16[ij]);
			}
		}
	}

	//Calculate a temporal average "ave[ij]" and a standard deviation "sigma[ij]" for each pixel.
	for (i = 0; i < ImageHeight; i++) {
		for (j = 0; j < ImageWidth; j++) {
			ij = ImageWidth*i + j;
			ave[ij]   = (double)x_sum[ij] / (double)packet_size;
			sigma[ij] = ( (double)x2_sum[ij] - (double)packet_size * ave[ij] * ave[ij] ) / (double)(packet_size - 1);
			sigma[ij] = sqrt( sigma[ij] );

		}
	}

	free (x_sum);	//Free the memory.
	free (x2_sum);

	return;
}

//Mask an area around a pixel brighter than "red_val".
void mask_bright()
{
	int i, j, ij;

	for (i = MARGIN; i < ImageHeight - MARGIN; i++) {						//"MARGIN" is the width along the 4 sides of the image (see header.h).
		for (j = MARGIN; j < ImageWidth - MARGIN; j++) {
			ij = ImageWidth*i + j;
			if( ave[ij] > (double)red_val  )  mask_on( i, j, mask_area1 );	//"ave[ij] is the temporal average of pixel values over frames in a packet.
		}																	//"mask_area1" defines the size of the mask area.
	}
	return;
}

//Mask an area around a pixel darker than "blue_val".
void mask_dark()
{
	int i, j, ij;

	for (i = MARGIN; i < ImageHeight - MARGIN; i++) {
		for (j = MARGIN; j < ImageWidth - MARGIN; j++) {
			ij = ImageWidth*i + j;
			if( ave[ij] < (double)blue_val )  mask_on( i, j, mask_area2 );	//"mask_area2" defines the size of the mask area.
		}
	}
	return;
}

//Mask an area around a star.
void mask_star()
{
	int i, j, ij;

	//According to "star_on[ij]" obtained in the previous packet, we apply masks.
	for (i = MARGIN; i < ImageHeight - MARGIN; i++) {
		for (j = MARGIN; j < ImageWidth - MARGIN; j++) {
			ij = ImageWidth*i + j;
			if( star_on[ij] ) mask_on( i, j, mask_area3 );	//"mask_area3" defines the size of the mask area.
		}
	}

	//Calculate "star_on[ij]" (if TRUE, there is a star) for the next packet.
	for (i = MARGIN; i < ImageHeight - MARGIN; i++) {
		for (j = MARGIN; j < ImageWidth - MARGIN; j++) {
			ij = ImageWidth*i + j;
			if( find_star( i, j ) ) star_on[ij] = TRUE;
			else star_on[ij] = FALSE;
		}
	}

	return;
}

//Check whether the point (i0, j0) is a star or not.
BOOL find_star( int i0, int j0 )
{
	int i, j, ij;
	int n = 0;
	double x_sum  = 0.;
	double x2_sum = 0.;

	int ij0 = ImageWidth*i0 + j0;

	for ( i = i0 - star_size; i <= i0 + star_size; i++ ) {	//"star_size" defines the square around the point (i0, j0).
		if( i < 0 || i >= ImageHeight ) continue;			//In case the coordinate "i" is out of the image;
		for ( j = j0 - star_size; j <= j0 + star_size; j++ ) {
			if( j < 0 || j >= ImageWidth ) continue;		//In case the coordinate "j" is out of the image;
			if( i == i0 - star_size || i == i0 + star_size || j == j0 - star_size || j == j0 + star_size ) {
				ij = ImageWidth*i + j;
				x_sum  += ave[ij];
				x2_sum += ave[ij] * ave[ij];
				n ++;
			}
		}
	}
	if( n < 2 ) return FALSE;	//In case "n" is 0 or 1, we cannot calculate standard variations and we don't apply the mask.

	double ave2   = x_sum / (double)n;											//The spatial average of pixel values on the 4 sides of the area.
	double sigma2 = ( x2_sum - (double)n * ave2 * ave2 ) / (double)( n - 1 );	//Dispersion
		   sigma2 = sqrt( sigma2 );												//Spatial standard deviation

	if( ( ave[ij0] - ave2 ) > sigma2 * (double)star_threshold/10. ) {					//If the center "ave[ij0]" is brighter than the spatial average by (star_threshold/10) times the spatial standard deviation "sigma2";
		if(
			( ( ave[ij0-1] - ave2 ) / ( ave[ij0] - ave2 ) * 100. >= (double)hidari )	//If all the adjacent pixels exceed some % of the brightness of the central pixel;
		&&	( ( ave[ij0+1] - ave2 ) / ( ave[ij0] - ave2 ) * 100. >= (double)migi )
		&&	( ( ave[ij0 - ImageWidth] - ave2 ) / ( ave[ij0] - ave2 ) * 100. >= (double)ue )
		&&	( ( ave[ij0 + ImageWidth] - ave2 ) / ( ave[ij0] - ave2 ) * 100. >= (double)shita )
		) return TRUE;		//Apply the mask
		else return FALSE;	//No mask
	}

	else return FALSE;
}

//Initialize "star_on[ij]" as there is no stars.
void star_off()
{
	int i, j, ij;

	for ( i = 0; i < ImageHeight; i++ ) {
		for ( j = 0; j < ImageWidth; j++ ) {
			ij = ImageWidth*i + j;
			star_on[ij] = FALSE;	//No star as an initial condition.
		}
	}

	return;
}

//Mask pixels by changing the "sigma[ij]" (a standard deviation image) to the impossibly large value "MASK_VALUE" (see header.h).
void mask_on( int i0, int j0, int mask_area )	//"i0" and "j0" are the coordinates of the center of the area to be masked.
{												//"mask_area" defines the size of the area.
	int i, j, ij;

	for ( i = i0 - mask_area; i <= i0 + mask_area; i++ ) {
		if( i < 0 || i >= ImageHeight ) continue;		//In case the coordinate "i" is out of the image.
		for ( j = j0 - mask_area; j <= j0 + mask_area; j++ ) {
			if( j < 0 || j >= ImageWidth ) continue;
			ij = ImageWidth*i + j;
			sigma[ij] = (double)MASK_VALUE;
		}
	}

	return;
}

//Mask the manually defined areas.
void mask_manual( POINTS mask_manual_TL, POINTS mask_manual_BR )	//Two arguments are the upper-left and the lower-right corners of the area.
{
	int i, j, ij, tmp;

	//In case the coordinates are out of the image;
	if( mask_manual_TL.x < 0 ) mask_manual_TL.x = 0;
	if( mask_manual_TL.x >= ImageWidth  ) mask_manual_TL.x = ImageWidth  - 1;
	if( mask_manual_TL.y < 0 ) mask_manual_TL.y = 0;
	if( mask_manual_TL.y >= ImageHeight ) mask_manual_TL.y = ImageHeight - 1;
	if( mask_manual_BR.x < 0 ) mask_manual_BR.x = 0;
	if( mask_manual_BR.x >= ImageWidth  ) mask_manual_BR.x = ImageWidth  - 1;
	if( mask_manual_BR.y < 0 ) mask_manual_BR.y = 0;
	if( mask_manual_BR.y >= ImageHeight ) mask_manual_BR.y = ImageHeight - 1;

	//In case the coordinates of the upper-left corner are larger than those of the lower-right corner;
	if( mask_manual_TL.x > mask_manual_BR.x ) {
		tmp = mask_manual_TL.x;
		mask_manual_TL.x = mask_manual_BR.x;
		mask_manual_BR.x = tmp;
	}
	if( mask_manual_TL.y > mask_manual_BR.y ) {
		tmp = mask_manual_TL.y;
		mask_manual_TL.y = mask_manual_BR.y;
		mask_manual_BR.y = tmp;
	}

	//Changing the "sigma[ij]" (a standard deviation image) to the impossible large value "MASK_VALUE" (see header.h).
	for ( i = mask_manual_TL.y; i <= mask_manual_BR.y; i++ ) {
		for ( j = mask_manual_TL.x; j <= mask_manual_BR.x; j++ ) {
			ij = ImageWidth*i + j;
			sigma[ij] = (double)MASK_VALUE;
		}
	}

	return;
}

//Search impact flashes for all frames in a packet.
BOOL search( int p_size )	//"p_size" is the number fo frames in the packet. Return TRUE when the number of frames that include flash candidates exceeds a pre-set value.
{
	int  i, j, ij, n;
	static int i_min, i_max;	//The minimum and the maximum of the x-coordinates of flash candidates in a frame.
	static int j_min, j_max;
	BOOL flag_detection;		//When a frame includes at least one flash candidate, "flag_detection" is TRUE.
	int  count_square = 0;		//The number of small squares that point to the candidates.
	char tmp[100];				//The coordinates, brightness, etc. of a candidate are stored here.
	static char str[1000];		//The coordinates, brightness, etc. of all candidates in a frame are stored here".
	const int strsize = 1000;

	_fseeki64 ( fp1, (__int64)HEADER_SIZE + (__int64)c_frame * (__int64)ImageSize * 2, SEEK_SET );	//Setting the pointer "fp1" at the top of the image data of "c_frame".

	//Frame by frame detection process in a packet.
	for ( n = 0; n < p_size; n++ ){
		flag_detection = FALSE;
		count_square = 0;

		//Detection process in a frame.
		ser_read( fp1 );		//Read a frame in the SER file.
		frame_display( FALSE );		//Copy the frame to "g_img.lpBmpData" for saving a BITMAP image; The display is not updated.
		i_min = 9999; i_max = -9999;	//Initialization to find the minimum and maximum x-coordinates.
		j_min = 9999; j_max = -9999;

		for( i = MARGIN; i < ImageHeight - MARGIN; i++ ) {		//We don't search for impact flashes in the marginal area of the frame where it may be noisy (see header.h for "MARGIN").
			for( j = MARGIN; j < ImageWidth - MARGIN; j++ ) {
				ij = ImageWidth*i +j;

				//Criteria for detection:
				if( ((double)value16[ij]*10.) > (ave[ij]*10. + sigma[ij] * (double)threshold) ) {	//Because "threshold" is 10 times the actual threshold value, we multiply both "value16" and "ave" by 10.
					if(
							( ( (double)value16[ij-1] - ave[ij-1] ) / ( (double)value16[ij] - ave[ij] ) * 100. >= (double)hidari )
						&&	( ( (double)value16[ij+1] - ave[ij+1] ) / ( (double)value16[ij] - ave[ij] ) * 100. >= (double)migi )
						&&	( ( (double)value16[ij - ImageWidth] - ave[ij - ImageWidth] ) / ( (double)value16[ij] - ave[ij] ) * 100. >= (double)ue )
						&&	( ( (double)value16[ij + ImageWidth] - ave[ij + ImageWidth] ) / ( (double)value16[ij] - ave[ij] ) * 100. >= (double)shita )
					)
					{	//When the criteria are satisfied;
						if( i < i_min ) i_min = i;	//A process to find the minimum of the x-coordinate.
						if( i > i_max ) i_max = i;
						if( j < j_min ) j_min = j;
						if( j > j_max ) j_max = j;
						count_square++;				//"count_square" is the number of small squares that point to the candidates in a frame.
						n_detection += 1;			//"n_detection" is the number of detections in a SER file.

						if( count_square > SQUARE_LIMIT ) continue;	//When the number of detections in a frame exceeds "SQUARE_LIMIT" (see headr.h), we don't record them in a log and don't draw squares in the frame, and continue the detection procedure.

						wsprintf ( tmp, "\n%-10d%-10d%-10d%-10d%-10d%-10d%-10d", n_detection, c_frame, j, i, (int)value16[ImageWidth*i + j], (int)ave[ImageWidth*i + j], (int)sigma[ImageWidth*i + j] );
						if( count_square <= 1 ) strcpy_s ( str, strsize, tmp );	//At the first detection in a frame, we copy "tmp" to the top of "str".
						else strcat_s ( str, strsize, tmp );					//At the successive detections, add "tmp" to "str".

						draw_square( i, j );		//Draw a square that points to the point of detection.
						flag_detection = TRUE;		//Show that there are impact flash candidates in the frame.
					}		
				}
			}
		}
		//The end of the detection process for a frame.

		//Procedure when there are impact flash candidates in the frame.
		if ( flag_detection == TRUE ) {
			if( count_square > SQUARE_LIMIT && ( i_max - i_min > scatter_i || j_max - j_min > scatter_j ) ) {	//When the number of detection in a frame exceeds "SQUARE_LIMIT" (see headr.h) and they scatter widely in a frame, we judge that all of them are false-positives.
				n_detection -= count_square;
				break;
			}
			fprintf( fp2, "%s", str );	//Save information on detections in a frame.
			bmp_save();					//Save a BITMAP image file of the frame.
			n_frames += 1;				//"n_frames" is the number of frames with detections in a SER file.

			if( n_frames >= detect_limit ) return TRUE;		//Stop the search procedure in a SER file, when the number of frames with detections exceeds the pre-set value.
		}

		c_frame++;		//"c_frame" is the successive frame number in a SER file.

	}
	//The end of the detection process in a packet.

	return FALSE;
}

//Save a frame that includes impact flash candidates as a BITMAP image file.
void bmp_save()
{
	char path_name2[100];	//A path-name for a BITMAP image file.
	FILE *fp;				//A pointer for a BITMAP image file.

	wsprintf(path_name2, "%s\\%04d_t%02d_%06d.bmp", detect_dir, n_frames, threshold, c_frame ); //Make up the path-name by combining informations.

	fopen_s( &fp, path_name2, "wb" );	//Open a file whose path-name is "path_name2" in a binary writing mode.

	//Prepare a file header of a BITMAP image file.
		BITMAPFILEHEADER bfh;
		bfh.bfType      = 'M'*256+'B';
		bfh.bfReserved1 = 0;
		bfh.bfReserved2 = 0;
		bfh.bfOffBits   = sizeof(BITMAPFILEHEADER)+ sizeof(BITMAPINFOHEADER);
		bfh.bfSize      = bfh.bfOffBits + g_img.bmp_ih.biSizeImage;

	fwrite( &bfh, sizeof(BITMAPFILEHEADER), 1, fp );				//Write the file header.
	fwrite( &( g_img.bmp_ih ), sizeof(BITMAPINFOHEADER), 1, fp );	//Write the information header of the BITMAP image file.
	fwrite( g_img.lpBmpData , 1, g_img.bmp_ih.biSizeImage,  fp );	//Write the data of the BITMAP image file.
	fclose( fp );													//Close the file.
	return;
}

//Draw a square in a BITMAP image as a pointer to an impact flash candidate.
void  draw_square( int i0, int j0 ){	//"i0" and "j0" are the coordinates of the candidate.
	int i, j, ij;

	int ij0 = ImageWidth*i0 + j0;

	for ( i = i0 - SQUARE_SIZE; i <= i0 + SQUARE_SIZE; i++ ) {
		for ( j = j0 - SQUARE_SIZE; j <= j0 + SQUARE_SIZE; j++ ) {
			if( i == i0 - SQUARE_SIZE || i == i0 + SQUARE_SIZE || j == j0 - SQUARE_SIZE || j == j0 + SQUARE_SIZE ) {
				ij = ImageWidth*i + j;
				g_img.lpBmpData[ij*4] = 0;		//Blue: 0
				g_img.lpBmpData[ij*4+1] = 200;	//Green: 200
				g_img.lpBmpData[ij*4+2] = 255;	//Red: 255
			}
		}
	}
	return;
}