#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <shlobj.h>	//For open a folder.
#include  <direct.h>
#include  <Windows.h>
#include  <Windowsx.h>
#include  <math.h>
#include  <Shlwapi.h>	//For modifying path names.
#pragma comment(lib, "shlwapi.lib")

///////////////////////////////////////////////
#include <windows.h>
#include <commctrl.h>
#include "resource.h"
#pragma comment(lib, "comctl32.lib")

///////////////////////////////////////////////
#include "resource.h"

#define HEADER_SIZE    178	//The size of the header record of a SER file (unit: byte).
#define LIST_ITEM_MAX   40	//The upper limit of the number of SER files in LIST2.
#define THRESHOLD_LIMIT 45	//The lower limit of the threshold for flash detection multiplied by 10 (45 means 4.5 times temporal standard deviation).
#define SQUARE_SIZE      8	//The pixel size of squares (side length) that show detected points.
#define SQUARE_LIMIT    10	//The upper limit of the squares that show detected points in a frame, and also the upper limit of log outputs. The value is also used in a criteion of false-positive detections.
#define MARGIN          10	//The width of the sides of a frame where the search process is not applied.
#define MASK_VALUE   65536	//(The maximum number of a 16-bit integer) + 1.

//The column widths in IDC_LIST2 (unit: pixel):
#define C_WIDTH0    20		//Marker that shows a SER file displayed in the image window.
#define C_WIDTH1   100		//SER file name
#define C_WIDTH2	65		//Number of frames
#define C_WIDTH3	40		//The frame# to start the scan.
#define C_WIDTH4	55		//The frame# to stop the scan.
#define C_WIDTH5	40		//The maximum number of frames in a SER file that contain detections.
#define C_WIDTH6	40		//The packet size (unit: frame)
#define C_WIDTH7	40		//The initial threshold times 10.
#define C_WIDTH8	40		//The step of the threshold times 10.
#define C_WIDTH9	40		//The brightness control for display in the image window.

//Default values:
#define FRAMES_DETECTED_MAX	200	//The upper limit of the number of frames in a SER file.
#define PACKET              200	//The packet size (unit: frame).
#define BLUE_VALUE          500	//The dark pixel value.
#define RED_VALUE          3800	//The bright pixel value.

//The definition of IMG0 structure:
typedef struct{
	HINSTANCE         hi0;
	HWND              hwin;
	BYTE              *lpBmpData;	//Image data of a BITMAP.
	BITMAPINFOHEADER  bmp_ih;
	char			  *fname;
	int				  fsize;
} IMG0;

//Functions:
ATOM InitApp0( HINSTANCE );			//This function creates a class for the image window
BOOL InitInstance0(HINSTANCE, int);	//creates the image window

BOOL	CALLBACK  dlgproc ( HWND, UINT, WPARAM, LPARAM );	//for the main dialog box
BOOL	CALLBACK  dlgproc2( HWND, UINT, WPARAM, LPARAM );	//for a small window for changing parameters
LRESULT CALLBACK  WndProc0( HWND, UINT, WPARAM, LPARAM );	//for the image window

char* file_open();				//This function gets the pathname of a file
char* read_header( FILE* );		//reads the header record (178 Byte) of a ser movie file, which contains the width, height, grayscale depth, and frame number of the file
void  ser_read( FILE* );		//reads frames in a ser file

void  prep_bmp_ih();			//prepares BITMAP information header for the display in the image window and file save
void  prep_for_display();		//prepares for the image window and a BITMAP image
void  frame_display( BOOL );	//shows a frame in the image window; the argument determines whether the bright and the dark pixels are shown in colour or not

void  LIST2_prep();					//This function prepares "LIST2" in the dialog box
int   LIST2_get_checkmarked_item();	//returns the item number (SER file number) in "LIST2" for which the left end column shows "1"
void  LIST2_set_checkmark( int );	//shows "1" in the left end column for a specified item number in "LIST2", and shows blank for the others; the argument is an item number
void  LIST2_deselect();				//makes "SELECTED" and "FOCUSED" states OFF for all SER files in "LIST2"
void  LIST2_add();					//adds the informations and settings of a selected SER file in "LIST1" to a new line in "LIST2"
BOOL  show_file( int, BOOL );		//displays a frame when a SER file name is double-clicked in "LIST2" and returns TRUE in the case of success
int   get_iSubItem( int );			//calculates a subitem number in "LIST2" from a double-click position; the argument is a x-coordinate in "List2"
int   get_item_value( int, int );	//gets a value in a cell in "LIST2"; the arguments are an item number and a subitem number
BOOL  check_start_end_packet( int, int );	//returns FALSE if a new value typed in a small dialog box for changing parameters is not appropriate; the arguments are an item number and a subitem number
int   c_frame_check( int );			//corrects "c_frame" if it is not appropriate; the argument is an item number
void  read_parameter( int );		//reads parameters for a scan from the dialog box; the argument is the sequential number of a SER file in "List2"
void  result_txtout1( char* );		//prints results of a scan of a SER file to a file; the argument is a pointer to the header text of a SER file
void  result_txtout2();				//prints columun headers for the results of a scan for each threshold value

void  image_ave_and_std();				//This function calculates a temporal average and a temporal standard deviation of each pixel over a packet of frames
void  mask_bright();					//masks an area around a pixel brighter than a specified value
void  mask_dark();						//masks an area around a pixel darker than a specified value
void  mask_star();						//masks an area around a star
BOOL  find_star( int, int );			//checks whether a star is there or not; the arguments are coordinates
void  star_off();						//initializes "star_on[ij]" as there is no stars
void  mask_on( int, int, int );			//masks pixels by changing the "sigma[ij]" (a standard deviation image) to the impossibly large value "MASK_VALUE" (see header.h); two arguments are central coordinates and the other specifies the masksize
void  mask_manual( POINTS, POINTS );	//masks manually defined areas; the arguments are coordinates of the upper-left and lower-right corners

BOOL  search( int );	//This function searches impact flashes for all the frames in a packet and returns TRUE when the number of frames of detection reaches a specified number; the argument is a packet size
void  bmp_save();							//saves a frame that includes impact flashes as a BITMAP image file
void  draw_square( int, int );				//draws a square in a BITMAP image as a pointer to an impact flash candidate; the arguments are coordinates
void  DrawRect(HWND, POINTS, POINTS, int);	//draws a rectangular in the image window

//Global variables:
extern char list_fname[50];						//a SER file name
extern char file_path[LIST_ITEM_MAX][MAX_PATH];	//pathnames of SER files [item#][number of characters]. MAX_PATH=260

extern HINSTANCE hInst;
extern HWND	hDlg, hImage0;							//handles of a dialog box and an image window
extern HWND hCheck_mask_bright, hCheck_mask_dark;	//handles of check buttons for masking bright and dark areas
extern HWND hCheck_mask_star;						//a handle of a check button for masking around stars
extern HWND hRadio_mask1, hRadio_mask2, hRadio_mask3, hRadio_mask4;	//handles of radio buttons for manual-mask-regions
extern HWND hCheck_disp_mask;										//a handle for showing manual-mask-regions

extern HWND   hList2;		//a handle of IDC_LIST2, a list box "List2"
extern LVITEM list_item;	//a List View Item structure
extern int	  list_file;	//the number of SER files in "LIST2"

extern IMG0 g_img;								//see header.h for IMG0
extern char szClassName0[];						//a class name of the image window

extern FILE *fp1;			//for SER files
extern FILE *fp2;			//for output text files

extern int bit_shift;			//"bit_shift=1, 2, 3" correspond to dividing a pixel value in 16-bit by 2, 4, 8 in reading SER files
extern unsigned short *value16;	//pixel values of a frame of a SER file are stored here
extern double *ave;				//a time-averaged image
extern double *sigma;			//a temporal standard deviation image
extern BOOL   *star_on;			//a BOOL image for masking pixels around stars

extern int ImageWidth, ImageHeight, PixelDepthPerPlane, FrameCount;	//the width, height, grayscale, and total number of frames of a SER file
extern int ImageWidth2, ImageHeight2, ImageSize;					//the width and height of images displayed in the image window, and the number of pixels in an original frame
extern int disp_size;												//the size of the image window : 1 for 100 % , 2 for 50 % , 3 for 33 % , and 4 for 25 %

extern char detect_dir[100];			//a name of a folder that contains outputs of scans
extern char detect_result_path[200];	//a pathname of a log file
extern char detect_dir_c_frame[100];	//for the new name of a folder that contains outputs of scans

extern int c_frame;									//a frame# in a SER file
extern int total_frame, start, end, detect_limit, packet_size, blue_val, red_val, mag_brt, threshold, d_threshold, hidari, migi, ue, shita;
extern int mask_area1, mask_area2, mask_area3;		//sizes of mask areas around bright, dark, and star pixels
extern int star_size, star_threshold;				//a size of background areas in searching stars, and a threshold in searching stars
extern POINTS mask_manual_TL[4], mask_manual_BR[4];	//top-left and bottom-right coordinates of manual-mask-areas
extern int scatter_i, scatter_j;					//upper limits of the scatter of y- and x-coordinates of the detected points
extern int n_detection, n_frames;					//"n_detection" is the number of detections in a SER file, and "n_frames" is the number of frames that contain detections

extern char	 buf_for_dlg2[256];	//used for a small dialog box
extern char  szBuf[256];		//for general purpose